import { isDefined } from './rtti';
import { each } from './enumeration';

var deferTimeoutIds = {};

function defer( operation, delay ) {
  var timeoutId = window.setTimeout(
    function() {
      delete deferTimeoutIds[ timeoutId ];
      operation();
    },
    delay || 0
  );
  deferTimeoutIds[ timeoutId ] = operation;

  return function() {
    if ( isDefined( deferTimeoutIds[ timeoutId ] ) ) {
      delete deferTimeoutIds[ timeoutId ];
      window.clearTimeout( timeoutId );
      return true;
    }
    return false;
  };
}

defer.flush = function() {
  var counter = 0;
  each( deferTimeoutIds, function( operation, deferTimeoutId ) {
    delete deferTimeoutIds[ deferTimeoutId ];
    counter++;
    operation();
  } );
  return counter;
};

export { defer };
