import { each } from './enumeration';
import { isDefined, isUndefined, isArray, isObject, isFunction } from './rtti';
import { noop } from './function';

var jQLite = {};

( function() {
  /*
      Simplified DOM selection engine
      START ---------------------------------------------------------
    */

  var parseSelector = function( selector, context ) {

    context = context || document;

    if ( selector.nodeType && selector.nodeType === DOM_DOCUMENT_NODE ) {
      selector = document.body;
      if ( selector === null ) {
        // Body not ready yet, return the document instead
        return [ document ];
      }
    }

    if ( selector.nodeType && selector.nodeType === DOM_ELEMENT_NODE ) {
      // Is the selector already a single DOM node?
      return [ selector ];
    }

    if ( selector.jquery && typeof selector.jquery === 'string' ) {
      // Is the selector a jQL object?
      return selector.toArray();
    }

    if ( context ) {
      context = cleanUp( context );
    }

    if ( jQL.isArray( selector ) ) {
      // This is already an array of nodes
      return selector;
    }
    else if ( typeof selector === 'string' ) {

      // This is the meat and potatoes
      var nodes = [];
      for ( var cN = 0; cN < context.length; cN++ ) {
        // For each context node, look for the
        // specified node within it
        var ctxNode = [ context[cN] ];
        if ( !jQL.forceSimpleSelectorEngine && ctxNode[0].querySelectorAll ) {
          var nl = ctxNode[0].querySelectorAll( selector );
          for ( var tni = 0; tni < nl.length; tni++ ) {
            nodes.push( nl.item( tni ) );
          }
        }
      }
      return nodes;
    }
    else {
      // What do you want me to do with this?
      return null;
    }
  };



  //------------------ EVENTS

  /**
    * Associative array of events and their types
    * @private
    */
  var EVENT_TYPES = {
    click:'MouseEvents', dblclick:'MouseEvents', mousedown:'MouseEvents',
    mouseup:'MouseEvents', mouseover:'MouseEvents', mousemove:'MouseEvents',
    ouseout:'MouseEvents', contextmenu:'MouseEvents', keypress:'KeyEvents',
    keydown:'KeyEvents', keyup:'KeyEvents', load:'HTMLEvents',
    unload:'HTMLEvents', abort:'HTMLEvents', error:'HTMLEvents',
    resize:'HTMLEvents', scroll:'HTMLEvents', select:'HTMLEvents',
    change:'HTMLEvents', submit:'HTMLEvents', reset:'HTMLEvents',
    focus:'HTMLEvents', blur:'HTMLEvents', touchstart:'MouseEvents',
    touchend:'MouseEvents', touchmove:'MouseEvents',
    pointermove:'PointerEvents', pointerout:'PointerEvents',
    pointerdown:'PointerEvents', pointerup:'PointerEvents'
  };

  var createEvent = function( eventType ) {
    if ( typeof eventType === 'string' ) {
      eventType = eventType.toLowerCase();
    }

    var evt = null;
    var eventClass = EVENT_TYPES[eventType] || 'Event';
    if ( document.createEvent ) {
      evt = document.createEvent( eventClass );
      evt._eventClass = eventClass;
      if ( eventType ) {
        evt.initEvent( eventType, true, true );
      }
    }

    if ( document.createEventObject ) {
      evt = document.createEventObject();
      if ( eventType ) {
        evt.type = eventType;
        evt._eventClass = eventClass;
      }
    }

    return evt;
  };

  var fireEvent = function( node, eventType, data ) {
    var evt = createEvent( eventType );
    if ( evt._eventClass !== 'Event' ) {
      evt.data = data;
      return node.dispatchEvent( evt );
    }
    else {
      var eHandlers = node._handlers || {};
      var handlers = eHandlers[eventType];
      if ( handlers ) {
        for ( var h = 0; h < handlers.length; h++ ) {
          var args = jQL.isArray( data ) ? data : [];
          args.unshift( evt );
          var op = handlers[h].apply( node, args );
          op = ( typeof op == 'undefined' ? true : op );
          if ( !op ) {
            break;
          }
        }
      }
    }
  };

  var setHandler = function( node, eventType, fn ) {
    if ( !jQL.isFunction( fn ) ) {
      return;
    }

    if ( typeof eventType === 'string' ) {
      eventType = eventType.toLowerCase();
    }

    var eventClass = EVENT_TYPES[eventType];
    if ( eventType.indexOf( 'on' ) === 0 ) {
      eventType = eventType.substring( 2 );
    }
    if ( eventClass ) {
      // Let the browser handle it
      var handler = function( evt ) {
        var aC = arguments.callee;
        var args = evt.data || [];
        args.unshift( evt );
        var op = aC.fn.apply( node, args );
        if ( typeof op != 'undefined' && op === false ) {
          if ( evt.preventDefault && evt.stopPropagation ) {
            evt.preventDefault();
            evt.stopPropagation();
          }
          else {
            evt.returnValue = false;
            evt.cancelBubble = true;
          }
          return false;
        }
        return true;
      };
      handler.fn = fn;
      if ( node.addEventListener ) {
        node.addEventListener( eventType, handler, false );
      }
      else {
        node.attachEvent( 'on' + eventType, handler );
      }
    }
    else {
      if ( !node._handlers ) {
        node._handlers = {};
      }
      var handlers = node._handlers[eventType] || [];
      handlers.push( fn );
      node._handlers[eventType] = handlers;
    }
  };
  /**
* jQuery "lite"
*
* This is a small subset of support for jQuery-like functionality.  It
* is not intended to be a full replacement, but it will provide some
* of the functionality which jQuery provides to allow development
* using jQuery-like syntax.
*/
  var jQL = function( s, e ) {
    return new jQLp().
      init( s, e );
  };

    
  var document = window.document;

    
  var hasOwnProperty = Object.prototype.hasOwnProperty;

    
  var toString = Object.prototype.toString;

    
  var push = Array.prototype.push;

    
  var slice = Array.prototype.slice;

    
  var DOM_ELEMENT_NODE = 1;

    
  var DOM_DOCUMENT_NODE = 9;

    
  var readyStack = [];

    
  var isReady = false;

    
  var setReady = false;

    
  var DOMContentLoaded;



  function int( str ) {
    return parseInt( str, 10 );
  }

  function normalizedUserAgent() {
    var userAgent = navigator.userAgent;
    if ( typeof userAgent == 'string' ) {
      userAgent = userAgent.toLowerCase();
    }
    return userAgent;
  }

  /**
 * IE 11 changed the format of the UserAgent string.
 * See http://msdn.microsoft.com/en-us/library/ms537503.aspx
 */
  var msie = int( ( /msie (\d+)/.exec( normalizedUserAgent() ) || [] )[1] );
  if ( isNaN( msie ) ) {
    msie = int( ( /trident\/.*; rv:(\d+)/.exec( normalizedUserAgent() ) || [] )[1] );
  }


  jQL.forceSimpleSelectorEngine = false;

  var stripScripts = function( data ) {
    // Wrap the data in a dom element
    var div = document.createElement( 'div' );
    div.innerHTML = data;
    // Strip out all scripts
    var scripts = div.getElementsByTagName( 'script' );

    return { scripts: scripts, data: data };
  };
  var properCase = function( str, skipFirst ) {
    skipFirst = skipFirst || false;
    str = ( !str ? '' : str.toString().
      replace( /^\s*|\s*$/g,'' ) );

    var returnString = '';
    if ( str.length <= 0 ) {
      return '';
    }

    var ucaseNextFlag = false;

    if ( !skipFirst ) {
      returnString += str.charAt( 0 ).
        toUpperCase();
    }
    else {
      returnString += str.charAt( 0 );
    }

    for ( var counter = 1; counter < str.length; counter++ ) {
      if ( ucaseNextFlag ) {
        returnString += str.charAt( counter ).
          toUpperCase();
      }
      else {
        returnString += str.charAt( counter ).
          toLowerCase();
      }
      var character = str.charCodeAt( counter );
      ucaseNextFlag = character === 32 || character === 45 || character === 46;
      if ( character === 99 || character === 67 ) {
        if ( str.charCodeAt( counter - 1 ) === 77 ||
          str.charCodeAt( counter - 1 ) === 109 ) {
          ucaseNextFlag = true;
        }
      }
    }
    return returnString;
  };
  var fixStyleProp = function( name ) {
    var tempName = name.replace( /-/g, ' ' );
    tempName = properCase( tempName, true );
    return tempName.replace( / /g, '' );
  };

  /**
* Loop over each object, performing the function for each one
* @param obj
* @param fn
*/
  jQL.each = function( obj, fn ) {
    var name; var i = 0;
    var length = obj.length;
    var isObj = length === undefined || jQL.isFunction( obj );

    if ( isObj ) {
      for ( name in obj ) {
        if ( fn.call( obj[ name ], name, obj[ name ] ) === false ) {
          break;
        }
      }
    }
    else {
      for ( var value = obj[0];
        i < length && fn.call( value, i, value ) !== false;
        value = obj[++i] ) {
        null;
      }
    }

    return obj;
  };

  /**
* NoOp function (empty)
*/
  jQL.noop = function() {};

  /**
* Test if the given object is a function
* @param obj
*/
  jQL.isFunction = function( obj ) {
    return toString.call( obj ) === '[object Function]';
  };

  /**
* Test if the given object is an Array
* @param obj
*/
  jQL.isArray = function( obj ) {
    return toString.call( obj ) === '[object Array]';
  };

  /**
* Test if the given object is an Object
* @param obj
*/
  jQL.isPlainObject = function( obj ) {
  // Make sure that DOM nodes and window objects don't pass through, as well
    if (
      !obj ||
      toString.call( obj ) !== '[object Object]' ||
      obj.nodeType ||
      obj.setInterval
    ) {
      return false;
    }

    // Not own constructor property must be Object
    if ( obj.constructor && !hasOwnProperty.call( obj, 'constructor' )
  && !hasOwnProperty.call( obj.constructor.prototype, 'isPrototypeOf' ) ) {
      return false;
    }

    // Own properties are enumerated firstly
    var key;
    for ( key in obj ) {
      null;
    }
    return key === undefined || hasOwnProperty.call( obj, key );
  };

  /**
* Merge two objects into one
* @param first
* @param second
*/
  jQL.merge = function( first, second ) {
    var i = first.length; var j = 0;

    if ( typeof second.length === 'number' ) {
      for ( var l = second.length; j < l; j++ ) {
        first[ i++ ] = second[ j ];
      }
    }
    else {
      while ( second[j] !== undefined ) {
        first[ i++ ] = second[ j++ ];
      }
    }

    first.length = i;

    return first;
  };

  jQL.param = function( params ) {
    var pList = '';
    if ( params ) {
      jQL.each( params, function( val, name ) {
        pList += ( pList.length !== 0 ? '&' : '' ) +
          name + '=' + encodeURIComponent( val );
      } );
    }
    return pList;
  };

  jQL.evalScripts = function( scripts ) {
    var head = document.getElementsByTagName( 'head' )[0] ||
      document.documentElement;
    for ( var s = 0; s < scripts.length; s++ ) {

      var script = document.createElement( 'script' );
      script.type = 'text/javascript';
      script.text = scripts[s].text;

      // Use insertBefore instead of appendChild to circumvent an IE6 bug.
      // This arises when a base node is used (#2709).
      head.insertBefore( script, head.firstChild );
      head.removeChild( script );
    }
  };

  jQL.ready = function() {
    isReady = true;
    while ( readyStack.length > 0 ) {
      var fn = readyStack.shift();
      fn();
    }
  };


  /////////////////////////////
  // cache from angularjs


  jQL.expando = 'um339';

  var jqCache = jQL.cache = {};
  var jqId = 1;
  var addEventListenerFn = ( window.document.addEventListener
    ? function( element, type, fn ) {
      element[0].addEventListener( type, fn, false );
    }
    : function( element, type, fn ) {
      element.attachEvent( 'on' + type, fn );
    } );
  var removeEventListenerFn = ( window.document.removeEventListener
    ? function( element, type, fn ) {
      element.removeEventListener( type, fn, false );
    }
    : function( element, type, fn ) {
      element.detachEvent( 'on' + type, fn );
    } );

  function jqNextId() {
    return ++jqId;
  }

  function indexOf( array, obj ) {
    if ( array.indexOf ) { 
      return array.indexOf( obj );
    }

    for ( var i = 0; i < array.length; i++ ) {
      if ( obj === array[i] ) {
        return i; 
      }
    }
    return -1;
  }
  function arrayRemove( array, value ) {
    var index = indexOf( array, value );
    if ( index >= 0 ) { 
      array.splice( index, 1 );
    }
    return value;
  }
  function jqLiteOff( type, fn, unsupported ) {
    var element = this;
    if ( isDefined( unsupported ) ) {
      throw Error(
        'offargs',
        'jqLite#off() does not support the `selector` argument' );
    }

    var events = jqLiteExpandoStore( element, 'events' );
    var handle = jqLiteExpandoStore( element, 'handle' );

    if ( !handle ) {
      return;
    } //no listeners registered

    if ( isUndefined( type ) ) {
      each( events, function( eventHandler, type ) {
        removeEventListenerFn( element, type, eventHandler );
        delete events[type];
      } );
    }
    else {
      each( type.split( ' ' ), function( type ) {
        if ( isUndefined( fn ) ) {
          removeEventListenerFn( element, type, events[type] );
          delete events[type];
        }
        else {
          arrayRemove( events[type] || [], fn );
        }
      } );
    }
  }


  function jqLiteExpandoStore( element, key, value ) {
    var expandoId = element.um339;

      
    var expandoStore = jqCache[expandoId || -1];

    if ( isDefined( value ) ) {
      if ( !expandoStore ) {
        element.um339 = expandoId = jqNextId();
        expandoStore = jqCache[expandoId] = {};
      }
      expandoStore[key] = value;
    }
    else {
      return expandoStore && expandoStore[key];
    }
  }

  /**
 * Creates a shallow copy of an object, an array or a primitive
 */
  function shallowCopy( src, dst ) {
    var i = 0;
    if ( isArray( src ) ) {
      dst = dst || [];

      for ( ; i < src.length; i++ ) {
        dst[i] = src[i];
      }
    }
    else if ( isObject( src ) ) {
      dst = dst || {};

      var keys = Object.keys( src );

      for ( var l = keys.length; i < l; i++ ) {
        var key = keys[i];

        if ( !( key.charAt( 0 ) === '$' && key.charAt( 1 ) === '$' ) ) {
          dst[key] = src[key];
        }
      }
    }

    return dst || src;
  }

  function createEventHandler( element, events ) {
    var eventHandler = function( event, type ) {
      if ( !event.preventDefault ) {
        event.preventDefault = function() {
          event.returnValue = false; //ie
        };
      }

      if ( !event.stopPropagation ) {
        event.stopPropagation = function() {
          event.cancelBubble = true; //ie
        };
      }

      if ( !event.target ) {
        event.target = event.srcElement || document;
      }

      if ( isUndefined( event.defaultPrevented ) ) {
        var prevent = event.preventDefault;
        event.preventDefault = function() {
          event.defaultPrevented = true;
          prevent.call( event );
        };
        event.defaultPrevented = false;
      }

      event.isDefaultPrevented = function() {
        return event.defaultPrevented || event.returnValue === false;
      };

      // Copy event handlers in case event handlers array
      // is modified during execution.
      var eventHandlersCopy = shallowCopy( events[type || event.type] || [] );

      each( eventHandlersCopy, function( fn ) {
        fn.call( element, event );
      } );

      // Remove monkey-patched methods (IE),
      // as they would cause memory leaks in IE8.
      if ( msie <= 8 ) {
      // IE7/8 does not allow to delete property on native object
        event.preventDefault = null;
        event.stopPropagation = null;
        event.isDefaultPrevented = null;
      }
      else {
      // It shouldn't affect normal browsers
      // (native methods are defined on prototype).
        delete event.preventDefault;
        delete event.stopPropagation;
        delete event.isDefaultPrevented;
      }
    };
    eventHandler.elem = element;
    return eventHandler;
  }

  var expando = 'jQuery' + ( +new Date ); var uuid = 0; var windowData = {};

  // The following elements throw uncatchable exceptions if you
  // attempt to add expando properties to them.
  jQL.noData = {
    'embed': true,
    'object': true,
    'applet': true
  };

  jQL.cache = {};

  jQL.data = function( elem, name, data ) {
    if ( elem.nodeName && jQL.noData[elem.nodeName.toLowerCase()] ) {
      return;
    }

    elem = elem === window ?
      windowData :
      elem;

    var id = elem[ expando ];

    // Compute a unique ID for the element
    if ( !id ) {
      id = elem[ expando ] = ++uuid;
    }

    // Only generate the data cache if we're
    // trying to access or manipulate it
    if ( name && !jQL.cache[ id ] ) {
      jQL.cache[ id ] = {};
    }

    // Prevent overriding the named cache with undefined values
    if ( data !== undefined ) {
      jQL.cache[ id ][ name ] = data;
    }
    else {
      data = name ?
        ( jQL.cache[ id ][ name ] ||
          ( jQL.cache[ id ][ name ] = elem.getAttribute( 'data-' + name ) ) ) :
        id;
    }

    return data;
  };

  jQL.removeData = function( elem, name ) {
    elem = elem === window ?
      windowData :
      elem;

    var id = elem[ expando ];

    // If we want to remove a specific section of the element's data
    if ( name ) {
      if ( jQL.cache[ id ] ) {
        // Remove the section of cache data
        delete jQL.cache[ id ][ name ];

        // If we've removed all the data, remove the element's cache
        name = '';

        for ( name in jQL.cache[ id ] ) {
          break;
        }

        if ( !name ) {
          jQL.removeData( elem );
        }
      }

      // Otherwise, we want to remove all of the element's data
    }
    else {
      // Clean up the element expando
      try {
        delete elem[ expando ];
      }
      catch ( e ) {
        // IE has trouble directly removing the expando
        // but it's ok with using removeAttribute
        if ( elem.removeAttribute ) {
          elem.removeAttribute( expando );
        }
      }

      // Completely remove the data cache
      delete jQL.cache[ id ];
    }
  };

  jQL.makeArray = function( array, results ) {
    var ret = results || [];
    if ( array != null ) {
      // The window, strings (and functions) also have 'length'
      // The extra typeof function check is to prevent crashes
      // in Safari 2 (See: #3039)
      if (
        array.length == null ||
        typeof array === 'string' ||
        isFunction( array ) ||
        ( typeof array !== 'function' && array.setInterval )
      ) {
        push.call( ret, array );
      }
      else {
        jQL.merge( ret, array );
      }
    }

    return ret;
  };

  jQL.inArray = function( e, arr ) {
    for ( var a = 0; a < arr.length; a++ ) {
      if ( arr[a] === e ) {
        return a;
      }
    }
    return -1;
  };

  jQL.trim = function( str ) {
    if ( str != null ) {
      return str.toString().
        replace( /^\s*|\s*$/g,'' );
    }
    else {
      return '';
    }
  };

  var jQLp = function() {};
  jQLp.prototype = {

    selector: '',
    context: null,
    length: 0,
    jquery: 'jqlite-1.1.1-unimatrix',

    init: function( s, e ) {

      if ( !s ) {
        return this;
      }

      if ( s.nodeType ) {
      // A simple node
        this.context = this[0] = s;
        this.length = 1;
      }
      else if ( typeof s === 'function' ) {
      // Short-form document.ready()
        this.ready( s );
      }
      else {
        var els = [];
        if ( s.jquery && typeof s.jquery === 'string' ) {
        // Already jQLite, just grab its elements
          els = s.toArray();
        }
        else if ( jQL.isArray( s ) ) {
        // An array of elements
          els = s;
        }
        else if ( typeof s === 'string' && jQL.trim( s ).
          indexOf( '<' ) === 0 && jQL.trim( s ).
          indexOf( '>' ) !== -1 ) {
        // This is most likely html, so create an element for them
          var elm = getParentElem( s );
          var h = document.createElement( elm );
          h.innerHTML = s;
          // Extract the element
          els = [ h.removeChild( h.firstChild ) ];
          h = null;
        }
        else {
          var selectors;
          if ( s.indexOf( ',' ) !== -1 ) {
          // Multiple selectors - split them
            selectors = s.split( ',' );
            for ( var n = 0; n < selectors.length; n++ ) {
              selectors[n] = jQL.trim( selectors[n] );
            }
          }
          else {
            selectors = [ s ];
          }

          var multi = [];
          for ( var m = 0; m < selectors.length; m++ ) {
            multi = multi.concat( parseSelector( selectors[m], e ) );
          }
          els = multi;
        }

        push.apply( this, els );

      }
      return this;
    },

    // CORE

    each: function( fn ) {
      return jQL.each( this, fn );
    },

    size: function() {
      return this.length;
    },

    toArray: function() {
      return slice.call( this, 0 );
    },

    ready: function( fn ) {
      if ( isReady ) {
        fn();
      }
      else {
        readyStack.push( fn );
        return this;
      }
    },

    data: function( key, value ) {
      if ( typeof key === 'undefined' && this.length ) {
        return jQL.data( this[0] );

      }
      else if ( typeof key === 'object' ) {
        return this.each( function() {
          jQL.data( this, key );
        } );
      }

      var parts = key.split( '.' );
      parts[1] = parts[1] ? '.' + parts[1] : '';

      if ( value === undefined ) {

        if ( typeof data === 'undefined' && this.length ) {
          var data = jQL.data( this[0], key );
        }
        return data === undefined && parts[1] ?
          this.data( parts[0] ) :
          data;
      }
      else {
        return this.each( function() {
          jQL.data( this, key, value );
        } );
      }
    },

    removeData: function( key ) {
      return this.each( function() {
        jQL.removeData( this, key );
      } );
    },

    replaceWith: function( replaceNode ) {
      each( this, function( thisNode ) {
        var index;
        var parent = thisNode.parentNode;

        each( new jQL( replaceNode ), function( node ) {
          if ( index ) {
            parent.insertBefore( node, index.nextSibling );
          }
          else {
            parent.replaceChild( node, thisNode );
          }
          index = node;
        } );
      } );

    },

    text: function( ) {
      var string = '';
      each( this, function( element ) {
        string += element.innerText;
      } );
      return string;
    },

    contents: function() {
      var contentsArray = [];
      each( this, function( element ) {
        var elementNodes = Array.prototype.slice.call( element.childNodes );
        contentsArray = contentsArray.concat( elementNodes );
      } );
      return contentsArray;
    },

    // CSS

    addClass: function( cName ) {
      return this.each( function() {
        if ( this.className.length !== 0 ) {
          var cn = this.className.split( ' ' );
          if ( jQL.inArray( cName, cn ) === -1 ) {
            cn.push( cName );
            this.className = cn.join( ' ' );
          }
        }
        else {
          this.className = cName;
        }
      } );
    },

    removeClass: function( cName ) {
      return this.each( function() {
        if ( this.className.length !== 0 ) {
          var cn = this.className.split( ' ' );
          var i = jQL.inArray( cName, cn );
          if ( i !== -1 ) {
            cn.splice( i, 1 );
            this.className = cn.join( ' ' );
          }
        }
      } );
    },

    hasClass: function( cName ) {
      if ( this[0].className.length === 0 ) {
        return false;
      }
      return jQL.inArray( cName, this[0].className.split( ' ' ) ) !== -1;
    },

    isElementName: function( eName ) {
      return ( this[0].nodeName.toLowerCase() === eName.toLowerCase() );
    },

    toggleClass: function( cName ) {
      return this.each( function() {
        if ( this.className.length === 0 ) {
          this.className = cName;
        }
        else {
          var cn = this.className.split( ' ' );
          var i = jQL.inArray( cName, cn );
          if ( i !== -1 ) {
            cn.splice( i, 1 );
          }
          else {
            cn.push( cName );
          }
          this.className = cn.join( ' ' );
        }
      } );
    },

    hide: function( fn ) {
      return this.each( function() {
        if ( this.style && this.style['display'] != null ) {
          if ( this.style['display'].toString() !== 'none' ) {
            this._oldDisplay = this.style['display'].toString() ||
              ( this.nodeName !== 'span' ? 'block' : 'inline' );
            this.style['display'] = 'none';
          }
        }
        if ( jQL.isFunction( fn ) ) {
          fn( this );
        }
      } );
    },

    show: function( fn ) {
      return this.each( function() {
        this.style['display'] = (
          (
            this._oldDisplay &&
            this._oldDisplay !== '' ?
              this._oldDisplay : null
          ) ||
            ( this.nodeName !== 'span' ? 'block' : 'inline' )
        );
        if ( jQL.isFunction( fn ) ) {
          fn( this );
        }
      } );
    },

    css: function( sel, val ) {
      if ( typeof sel === 'string' && val == null ) {
        return this[0].style[fixStyleProp( sel )];
      }
      else {
        sel = typeof sel === 'string' ? makeObj( sel,val ) : sel;
        return this.each( function() {
          var self = this;
          if ( typeof self.style != 'undefined' ) {
            jQL.each( sel, function( key,value ) {
              value = ( typeof value === 'number' ? value + 'px' : value );
              var sn = fixStyleProp( key );
              if ( !self.style[sn] ) {
                sn = key;
              }
              self.style[sn] = value;
            } );
          }
        } );
      }
    },

    getStyle: function( sel ) {
    // warning, ie8 uses currentStyle which may be buggy
    // for some styles (things with non-px values)

      if ( window.getComputedStyle ) {
        return window.getComputedStyle( this[0],null ).
          getPropertyValue( sel );
      }
      else if ( document.documentElement.currentStyle ) {
        return this[0].currentStyle && this[0].currentStyle[ sel ];
      }
    },

    find: function( selector ) {
      var collections = [];
      each( this, function( element ) {
        if ( element.getElementsByTagName ) {
          var subcollection = element.getElementsByTagName( selector );
          each( subcollection, function( sub ) {
            collections.push( sub );
          } );
        }
      } );
      return jQL( collections );
    },


    clone: function() {
      var cloneElement = new jQL();
      each( this, function( itemNode, key ) {
        cloneElement[key] = itemNode.cloneNode( true );
        cloneElement.length++;
      } );
      return cloneElement;
    },

    wrap: function( wrapNode ) {
      wrapNode = jQL( wrapNode )[0];
      var element = this;
      each( element, function( itemNode ) {
        var parent = itemNode.parentNode;
        if ( parent ) {
          parent.replaceChild( wrapNode, itemNode );
        }
        wrapNode.appendChild( itemNode );
      } );

    },

    // HTML

    html: function( h ) {
      if ( !h ) {
        return this[0].innerHTML;
      }
      else {
        return this.each( function() {
          var o = stripScripts( h );
          this.innerHTML = o.data;
          jQL.evalScripts( o.scripts );
        } );
      }
    },

    attr: function( name, value ) {
      if ( typeof name === 'string' && value == null ) {
        if ( this[0] ) {
          return this[0].getAttribute( name );
        }
        else {
          return '';
        }
      }
      else {
        return this.each( function() {
          name = typeof name === 'string' ? makeObj( name,value ) : name;
          for ( var i in name ) {
            var v = name[i];
            this.setAttribute( i,v );
          }
        } );
      }
    },

    eq: function( index ) {
      var elms = this.toArray();
      var elm = index < 0 ? elms[elms.length + index] : elms[index];
      this.context = this[0] = elm;
      this.length = 1;
      return this;
    },

    first: function() {
      var elms = this.toArray();
      this.context = this[0] = elms[0];
      this.length = 1;
      return this;
    },

    last: function() {
      var elms = this.toArray();
      this.context = this[0] = elms[elms.length - 1];
      this.length = 1;
      return this;
    },

    index: function( selector ) {
      var idx = -1;
      if ( this.length !== 0 ) {
        var itm = this[0];
        if ( !selector ) {
          var parent = this.parent();
          var s = parent[0].firstChild;
          var arr = [];
          while ( s != null ) {
            if ( s.nodeType === DOM_ELEMENT_NODE ) {
              arr.push( s );
            }
            s = s.nextSibling;
          }
          jQL.each( s, function( i ) {
            if ( this === itm ) {
              idx = i;
              return false;
            }
          } );
        }
        else {
          var elm = jQL( selector )[0];
          this.each( function( i ) {
            if ( this === elm ) {
              idx = i;
              return false;
            }
          } );
        }
      }
      return idx;
    },

    next: function( selector ) {
      var arr = [];
      if ( !selector ) {
        this.each( function() {
          var elm = this.nextSibling;
          while ( elm != null && elm.nodeType !== DOM_ELEMENT_NODE ) {
            elm = elm.nextSibling;
          }
          if ( elm != null ) {
            arr.push( elm );
          }
        } );
      }
      else {
        var pElm = jQL( selector );
        this.each( function() {
          var us = this.nextSibling;
          while ( us != null && us.nodeType !== DOM_ELEMENT_NODE ) {
            us = us.nextSibling;
          }
          if ( us != null ) {
            var found = false;
            pElm.each( function() {
              if ( this === us ) {
                found = true;
                return false;
              }
            } );
            if ( found ) {
              arr.push( us );
            }
          }
        } );
      }
      return jQL( arr );
    },

    prev: function( selector ) {
      var arr = [];
      if ( !selector ) {
        this.each( function() {
          var elm = this.previousSibling;
          while ( elm != null && elm.nodeType !== DOM_ELEMENT_NODE ) {
            elm = elm.previousSibling;
          }
          if ( elm != null ) {
            arr.push( elm );
          }
        } );
      }
      else {
        var pElm = jQL( selector );
        this.each( function() {
          var us = this.previousSibling;
          while ( us != null && us.nodeType !== DOM_ELEMENT_NODE ) {
            us = us.previousSibling;
          }
          if ( us != null ) {
            var found = false;
            pElm.each( function() {
              if ( this === us ) {
                found = true;
                return false;
              }
            } );
            if ( found ) {
              arr.push( us );
            }
          }
        } );
      }
      return jQL( arr );
    },

    parent: function( selector ) {
      var arr = [];
      if ( !selector ) {
        this.each( function() {
          arr.push( this.parentNode );
        } );
      }
      else {
        var pElm = jQL( selector );
        this.each( function() {
          var us = this.parentNode;
          var found = false;
          pElm.each( function() {
            if ( this === us ) {
              found = true;
              return false;
            }
          } );
          if ( found ) {
            arr.push( us );
          }
        } );
      }
      return jQL( arr );
    },

    parents: function( selector ) {
      var arr = [];
      if ( !selector ) {
        this.each( function() {
          var us = this;
          while ( us !== document.body ) {
            us = us.parentNode;
            arr.push( us );
          }
        } );
      }
      else {
        var pElm = jQL( selector );
        this.each( function() {
          var us = this;
          while ( us !== document.body ) {
            pElm.each( function() {
              if ( this === us ) {
                arr.push( us );
              }
            } );
            us = us.parentNode;
          }
        } );
      }
      return jQL( arr );
    },

    children: function( selector ) {
      var arr = [];
      if ( !selector ) {
        this.each( function() {
          var us = this.firstChild;
          while ( us != null ) {
            if ( us.nodeType === DOM_ELEMENT_NODE ) {
              arr.push( us );
            }
            us = us.nextSibling;
          }
        } );
      }
      else {
        var cElm = jQL( selector );
        this.each( function() {
          var us = this.firstChild;
          while ( us != null ) {
            if ( us.nodeType === DOM_ELEMENT_NODE ) {
              cElm.each( function() {
                if ( this === us ) {
                  arr.push( us );
                }
              } );
            }
            us = us.nextSibling;
          }
        } );
      }
      return jQL( arr );
    },

    isDescendantOf: function( parent ) {
      parent = jQL( parent )[0];
      var node = jQL( this )[0] ? jQL( this )[0].parentNode : null;
      while ( node != null ) {
        if ( node === parent ) {
          return true;
        }
        node = node.parentNode;
      }
      return false;
    },

    isAncestorOf: function( child ) {
      var parent = jQL( this )[0];
      var node = child ? jQL( child )[0].parentNode : null;
      while ( node != null ) {
        if ( node === parent ) {
          return true;
        }
        node = node.parentNode;
      }
      return false;
    },

    append: function( child ) {
      child = cleanUp( child );
      return this.each( function() {
        for ( var i = 0; i < child.length; i++ ) {
          this.appendChild( child[i] );
        }
      } );
    },


    prepend: function( nodes ) {
      each( this,function( element ) {
        if ( element.nodeType === 1 ) {
          var index = element.firstChild;
          each( nodes, function( child ) {
            element.insertBefore( child, index );
          } );
        }
      } );
      return ( this );
    },

    remove: function( els ) {
      return this.each( function() {
        if ( els ) {
          jQL( els, this ).
            remove();
        }
        else {
          var par = this.parentNode;
          if ( par ) {
            par.removeChild( this );
          }
        }
      } );
    },

    after: function( newElement ) {
      each( this, function( element ) {
        var index = element; var parent = element.parentNode;
        each( new jQL( newElement ), function( node ) {
          parent.insertBefore( node, index.nextSibling );
          index = node;
        } );
      } );
    },

    empty: function() {
      return this.each( function() {
        this.innerHTML = '';
      } );
    },

    val: function( value ) {
      if ( value == null ) {
        var v = null;
        if (
          this &&
          this.length !== 0 &&
          typeof this[0].value !== 'undefined'
        ) {
          v = this[0].value;
        }
        return v;
      }
      else {
        return this.each( function() {
          if ( typeof this.value != 'undefined' ) {
            this.value = value;
          }
        } );
      }
    },

    // EVENTS

    on: function onFn( type, fn, unsupported ) {
      var element = this;
      if ( isDefined( unsupported ) ) {
        throw Error(
          'onargs',
          'jqLite#on() does not support the `selector`' +
          'or `eventData` parameters'
        );
      }

      var events = jqLiteExpandoStore( element, 'events' );


      var handle = jqLiteExpandoStore( element, 'handle' );

      if ( !events ) {
        jqLiteExpandoStore( element, 'events', events = {} );
      }
      if ( !handle ) {
        jqLiteExpandoStore(
          element,
          'handle',
          handle = createEventHandler( element, events )
        );
      }

      each( type.split( ' ' ), function( type ) {
        var eventFns = events[type];

        if ( !eventFns ) {
          if ( type === 'mouseenter' || type === 'mouseleave' ) {
            var contains = document.body.contains
            || document.body.compareDocumentPosition ?
              function( a, b ) {
                // jshint bitwise: false
                var adown = a.nodeType === 9 ? a.documentElement : a;
                var bup = b && b.parentNode;
                return a === bup || !!( bup && bup.nodeType === 1 && (
                  adown.contains ?
                    adown.contains( bup ) :
                    a.compareDocumentPosition &&
                      a.compareDocumentPosition( bup ) & 16
                ) );
              } :
              function( a, b ) {
                if ( b ) {
                  while ( ( b = b.parentNode ) ) {
                    if ( b === a ) {
                      return true;
                    }
                  }
                }
                return false;
              };

            events[type] = [];

            // Refer to jQuery's implementation of mouseenter & mouseleave
            // Read about mouseenter and mouseleave:
            // http://www.quirksmode.org/js/events_mouse.html#link8
            var eventmap = {
              mouseleave : 'mouseout',
              mouseenter : 'mouseover'
            };

            element.on( eventmap[type], function( event ) {
              var target = this; var related = event.relatedTarget;
              // For mousenter/leave call the handler if
              // related is outside the target.
              // NB: No relatedTarget if the mouse
              // left/entered the browser window
              if (
                !related ||
                ( related !== target && !contains( target, related ) )
              ) {
                handle( event, type );
              }
            } );

          }
          else {
            addEventListenerFn( element, type, handle );
            events[type] = [];
          }
          eventFns = events[type];
        }
        eventFns.push( fn );
      } );
    },

    off: jqLiteOff,

    one: function( type, fn ) {
      var element = jQL( this );

      //add the listener twice so that when it is called
      //you can remove the original function and still be
      //able to call element.off(ev, fn) normally
      element.on( type, function onFn() {
        element.off( type, fn );
        element.off( type, onFn );
      } );
      element.on( type, fn );
    },

    trigger: function( eType, data ) {
      return this.each( function() {
        return fireEvent( this, eType, data );
      } );
    },

    triggerHandler: function( eventName, eventData ) {
      this.each( function( element ) {
        var eventFns = (
          jqLiteExpandoStore( element, 'events' ) || {}
        )[eventName];

        eventData = eventData || [];

        var event = [ {
          preventDefault: noop,
          stopPropagation: noop
        } ];

        each( eventFns, function( fn ) {
          fn.apply( element, event.concat( eventData ) );
        } );

      } );
    },

    removeAttr: function( name ) {
      each( this, function( element ) {
        element.removeAttribute( name );
      } );
      return this;
    },

    prop: function( name, value ) {
      var propertyValue;
      each( this, function( element ) {
        if ( isDefined( value ) ) {
          element[name] = value;
        }
        else {
          propertyValue = element[name];
          return false;
        }
      } );
      return propertyValue;
    },

    submit: function( fn ) {
      return this.each( function() {
        if ( jQL.isFunction( fn ) ) {
          setHandler( this, 'onsubmit', fn );
        }
        else {
          if ( this.submit ) {
            this.submit();
          }
        }
      } );
    }

  };

  jQLp.prototype.bind = jQLp.prototype.on;
  jQLp.prototype.unbind = jQLp.prototype.off;

  // Cleanup functions for the document ready method
  if ( document.addEventListener ) {
    DOMContentLoaded = function() {
      document.removeEventListener(
        'DOMContentLoaded',
        DOMContentLoaded,
        false
      );
      jQL.ready();
    };

  }
  else if ( document.attachEvent ) {
    DOMContentLoaded = function() {
      // Make sure body exists, at least, in case IE gets a
      // little overzealous (ticket #5443).
      if ( document.readyState === 'complete' ) {
        document.detachEvent( 'onreadystatechange', DOMContentLoaded );
        jQL.ready();
      }
    };
  }

  var makeObj = function( sel, val ) {
    var o = {};
    o[sel] = val;
    return o;
  };

  var cleanUp = function( els ) {
    if ( els.nodeType && ( els.nodeType === DOM_ELEMENT_NODE ||
             els.nodeType === DOM_DOCUMENT_NODE ) ) {
      els = [ els ];
    }
    else if ( typeof els === 'string' ) {
      els = jQL( els ).
        toArray();
    }
    else if ( els.jquery && typeof els.jquery === 'string' ) {
      els = els.toArray();
    }
    return els;
  };

  var getParentElem = function( str ) {
    var s = jQL.trim( str ).
      toLowerCase();
    return s.indexOf( '<option' ) === 0 ? 'SELECT' :
      s.indexOf( '<li' ) === 0 ? 'UL' :
        s.indexOf( '<tr' ) === 0 ? 'TBODY' :
          s.indexOf( '<td' ) === 0 ? 'TR' : 'DIV';
  };

  jQLite = jQL;
  jQLite.fn = jQLp.prototype;

  // Document Ready
  if ( !setReady ) {
    setReady = true;
    // Catch cases where $(document).ready() is called after the
    // browser event has already occurred.

    if ( document.readyState === 'complete' ) {
      return jQL.ready();
    }

    // Mozilla, Opera and webkit nightlies currently support this event
    if ( document.addEventListener ) {
    // Use the handy event callback
      document.addEventListener( 'DOMContentLoaded', DOMContentLoaded, false );

      // A fallback to window.onload, that will always work
      window.addEventListener( 'load', jQL.ready, false );

    // If IE event model is used
    }
    else if ( document.attachEvent ) {
    // ensure firing before onload,
    // maybe late but safe also for iframes
      document.attachEvent( 'onreadystatechange', DOMContentLoaded );

      // A fallback to window.onload, that will always work
      window.attachEvent( 'onload', jQL.ready );
    }
  }

} )();

export { jQLite as element };
