import { each } from './enumeration';
import { isObject } from '../functions/rtti';

function extend( destination ) {
  each( arguments, function( object ) {
    if ( object !== destination ) {
      each( Object.keys( object ), function( key ) {
        destination[ key ] = object[ key ];
      } );
    }
  } );

  return destination;

}

function merge() {
  var result = {};

  function _merge( destination, source ) {
    for ( var property in source ) {
      if ( source[ property ] &&
           source[ property ].constructor &&
           source[ property ].constructor === Object ) {
        destination[ property ] = isObject( destination[ property ] ) ?
          destination[ property ] :
          {};

        merge( destination[ property ], source[ property ] );
      }
      else if ( source.hasOwnProperty( property ) ) {
        destination[ property ] = source[ property ];
      }
    }

    return destination;
  }

  if ( arguments.length === 1 ) {
    _merge( result, arguments[ 0 ] );
  }
  else if ( arguments.length > 1 ) {
    result = arguments[ 0 ];
    for ( var index = 1; index < arguments.length; index++ ) {
      _merge( result, arguments[ index ] );
    }
  }

  return result;
}

function getValue( object, path ) {
  var pathArr = path.split( '.' );
  var pathLength = pathArr.length;

  for ( var index = 0; index < pathLength; index++ ) {
    object = object[ path [ index ] ];
    if ( !object ) {
      return undefined;
    }
  }

  return object;
}

function deepCopy( o ) {
  var newO;
  var i;

  if ( typeof o !== 'object' ) {
    return o;
  }
  if ( !o ) {
    return o;
  }

  if ( '[object Array]' === Object.prototype.toString.apply( o ) ) {
    newO = [];
    for ( i = 0; i < o.length; i += 1 ) {
      newO[i] = deepCopy( o[i] );
    }
    return newO;
  }

  newO = {};
  for ( i in o ) {
    if ( o.hasOwnProperty( i ) ) {
      newO[i] = deepCopy( o[i] );
    }
  }
  return newO;
}

export { extend, merge, getValue, deepCopy };