function isDefined( value ) {
  return typeof value != 'undefined';
}

function isUndefined( value ) {
  return typeof value == 'undefined';
}

function isString( value ) {
  return typeof value === 'string';
}

function isObject( value ) {
  return value !== null && typeof value === 'object';
}

function isArray( value ) {
  if ( value == null || !value.constructor ) {
    return false;
  }
  return ( value.constructor === Array );
}

function isArrayLike( value ) {
  if ( value == null || isWindow( value ) ) {
    return false;
  }

  var length = value.length;

  if ( value.nodeType === 1 && length ) {
    return true;
  }

  return isString( value ) ||
          isArray( value ) ||
          length === 0 ||
          typeof length === 'number' && length > 0 && ( length - 1 ) in value;
}

function isWindow( value ) {
  return value && value.window === value;
}

function isFile( value ) {
  return ( value instanceof File );
}

function isBlob( value ) {
  return ( value instanceof Blob );
}

function isBoolean( value ) {
  return typeof value === 'boolean';
}

function isBlank( value ) {
  if ( isNumber( value ) ) {
    return false;
  }
  if ( isString( value ) ) {
    value = value.replace( /\s/g, '' );
    if ( !value.length ) {
      return true;
    }
    else {
      return false;
    }
  }
  if ( isArray( value ) ) {
    return !value.length;
  }
  if ( isObject( value ) ) {
    return !Object.entries( value ).length;
  }

  return !value;
}

function isRegExp( value ) {
  return ( value instanceof RegExp );
}

function isFunction( value ) {
  return typeof value === 'function';
}

function isAnnotatedFunction( value ) {
  return isArray( value ) && isFunction( value[ value.length - 1 ] );
}

function isDate( value ) {
  return ( value instanceof Date );
}

function isNumber( value ) {
  return !isNaN( parseFloat( value ) ) && isFinite( value );
}

export {
  isDefined,
  isUndefined,
  isString,
  isObject,
  isArray,
  isArrayLike,
  isWindow,
  isFile,
  isBlob,
  isBoolean,
  isBlank,
  isRegExp,
  isFunction,
  isAnnotatedFunction,
  isDate,
  isNumber
};
