import { isString, isDefined, isFunction, isNumber } from './rtti';

function lowerCase( string ) {
  return isString( string ) ? string.toLowerCase() : string;
}

function upperCase( string ) {
  return isString( string ) ? string.toUpperCase() : string;
}

function snakeCase( string ) {
  return ( string || '' ).
    replace( /^\s+|\s+$/gm, '' ).
    replace( /([^A-Z][A-Z])|([^\s-][\s-]+[^\s-])/g, function( match ) {
      return match.charAt( 0 ) + '_' + match.slice( -1 );
    } ).
    replace( /[_]+/g, '_' ).
    toLowerCase();
}

function camelCase( string ) {
  return snakeCase( string ).
    replace( /(^|[\s_-]+)[a-z]/g, function( match ) {
      return match.slice( -1 ).
        toUpperCase();
    } );
}

function lowerCamelCase( string ) {
  return snakeCase( string ).
    replace( /([\s_-]+)[a-z]/g, function( match ) {
      return match.slice( -1 ).
        toUpperCase();
    } );
}

function dashCase( string ) {
  return snakeCase( string ).
    replace( /([\s_]+)/g, '-' );
}

function pluralize( string ) {
  if ( string.charAt( string.length - 1 ) === 'y' ) {
    return string.slice( 0, -1 ) + 'ies';
  }
  else {
    return string + 's';
  }
}

function singularize( string ) {
  var str = string;

  if ( str.slice( -3, str.length ) === 'ies' ) {
    var str2 = str.slice( 0, str.length - 3 );
    return str2 + 'y';
  }
  else {
    return str.replace( /[sS]$/, '' );
  }
}

function digest( string ) {
  if ( string.length === 0 ) {
    return 0;
  }

  var result = 0x811c9dc5;

  var length = string.length;
  for ( var index = 0; index < length; index++ ) {
    result ^= string.charCodeAt( index );
    result += ( result << 1 ) +
      ( result << 4 ) +
      ( result << 7 ) +
      ( result << 8 ) +
      ( result << 24 );
  }

  return result >>> 0;
}

function padLeft( string, size, padCharacter ) {
  var pad;

  // convert integers, etc. to strings
  if (
    isDefined( string ) &&
    string !== null &&
    isFunction( string.toString )
  ) {
    string = string.toString();
  }

  if ( isNumber( size ) ) {
    size = parseInt( size );
    if ( isString( padCharacter ) ) {
      pad = Array( size + 1 ).
        join( padCharacter[0] );
    }
    else {
      pad = Array( size + 1 ).
        join( ' ' );
    }
  }
  else if ( isString( size ) ) {
    pad = size;
  }
  else {
    return string;
  }

  if ( !isString( string ) ) {
    return pad;
  }
  else if ( string.length >= pad.length ) {
    return string;
  }
  else {
    return ( pad + string ).slice( -pad.length );
  }

}

function padRight( string, size, padCharacter ) {
  var pad;

  // convert integers, etc. to strings
  if (
    isDefined( string ) &&
    string !== null &&
    isFunction( string.toString )
  ) {
    string = string.toString();
  }

  if ( isNumber( size ) ) {
    size = parseInt( size );
    if ( isString( padCharacter ) ) {
      pad = Array( size + 1 ).
        join( padCharacter[0] );
    }
    else {
      pad = Array( size + 1 ).
        join( ' ' );
    }
  }
  else if ( isString( size ) ) {
    pad = size;
  }
  else {
    return string;
  }

  if ( !isString( string ) ) {
    return pad;
  }
  else if ( string.length >= pad.length ) {
    return string;
  }
  else {
    return ( string + pad ).substring( 0, pad.length );
  }
}

export {
  lowerCase,
  upperCase,
  camelCase,
  lowerCamelCase,
  dashCase,
  pluralize,
  singularize,
  digest,
  padLeft,
  padRight
};
