import { module } from '../../unimatrix';
import { extend } from '../../functions/object';
import { isArray } from '../../functions/rtti';
import { each } from '../../functions/enumeration';
import { noop } from '../../functions/function';

module( 'Unimatrix' ).
  service(
    'Context',
    [
      function( ) {

        //---------------------------------------------------------------------
        // implementation

        var instanceNumber = 0;

        //---------------------------------------------------------------------
        // instance initialization

        function Context() {
          this.$$children = [];
          this.$$notificationObservers = {};
          this.$$id = instanceNumber;
        }

        //---------------------------------------------------------------------
        // instance operations

        Context.prototype = {

          'constructor': Context,

          create: function() {

            var newContext = Object.create( this );
            extend( newContext, this );
            newContext.$$children = [];
            newContext.$$notificationObservers = {};
            newContext.$$id = ++instanceNumber;

            this.$$children.push( newContext );
            return newContext;
          },

          subscribe: function( eventName, callback ) {
            if ( !isArray( this.$$notificationObservers[ eventName ] ) ) {
              this.$$notificationObservers[ eventName ] = [];
            }
            this.$$notificationObservers[eventName].push( callback );
          },

          unsubscribe: function( eventName, callback ) {
            // left as an excersize for the reader
            for ( var event in this.$$notificationObservers[eventName] ) {
              if ( event === callback ) {
                noop();
              }
            }
          },

          notify: function( eventName, payload ) {
            var self = this;
            each(
              this.$$notificationObservers[ eventName ] || [],
              function( observer ) {
                observer.apply( self, payload );
              }
            );
            each(
              this.$$children,
              function( context ) {
                context.notify( eventName, payload );
              }
            );
          },

          destroy: function() {
            each(
              this.$$children,
              function( context ) {
                context.destroy();
              }
            );
            this.notify( 'destroy' );
            this.$$children = [];
          }
        };

        return new Context();
      }
    ]
  );
