import { module } from '../../unimatrix';
import { noop } from '../../functions/function';
import { isDefined, isObject } from '../../functions/rtti';

module( 'Unimatrix' ).
  service(
    'Cookie',
    [
      function() {
        var packCookie = function( key, value, options ) {
          return [
            encodeURIComponent( key ),
            '=',
            encodeURIComponent( value ),
            isDefined( options.expiresOn ) ?
              '; expires=' + options.expiresOn.toUTCString() : '',
            isDefined( options.path ) ?
              '; path=' + options.path : '',
            isDefined( options.domain ) ?
              '; domain=' + options.domain : '',
            isDefined( options.secure ) ?
              '; secure' : ''
          ].join( '' );
        };

        var Cookie = function( key, value, options ) {
          options = options || {};
          var result = value;

          if ( isDefined( value ) ) {
            value = isObject( value ) ?
              JSON.stringify( value ) :
              String( value );
            document.cookie = packCookie( key, value, options );
          }
          else {
            result = Cookie.all()[ key ];
          }

          return result;

        };

        Cookie.all = function() {
          var cookieString = document.cookie;
          var cookiePattern = /([^=]+)[=]([^;]+);?(?:[\s]*)?/g;

          var cookies = {};
          var cookieMatch = null;
          while ( ( cookieMatch = cookiePattern.exec( cookieString ) ) ) {
            var key = cookieMatch[ 1 ];
            var value = decodeURIComponent( cookieMatch[ 2 ] );
            try {
              value = JSON.parse( value );
            }
            catch ( error ) {
              noop();
            }
            cookies[ key ] = value;
          }

          return cookies;
        };

        Cookie.remove = function( key ) {
          // is there a cookie with the given key?
          if ( isDefined( Cookie( key ) ) ) {
            // if so, remove it by setting it to expire in the past.
            Cookie(
              key,
              -1,
              { expiresOn: new Date( 'Thu, 01 Jan 1970 00:00:00 GMT' ) }
            );
          }
        };

        return Cookie;
      }
    ]
  );
