// Adapted from the angular.js $interpolate service
import { module } from '../../unimatrix';
import { isUndefined, isDefined } from '../../functions/rtti';
import { extend } from '../../functions/object';
import { valueFn } from '../../functions/function';

module( 'Unimatrix' ).
  service(
    'Interpolate',
    [ 'Unimatrix/Parser',
      function( Parser ) {

        var startSymbol = '{{';
        var endSymbol = '}}';
        var startSymbolLength = startSymbol.length;
        var endSymbolLength = endSymbol.length;
        var escapedStartRegexp = new RegExp( startSymbol.replace( /./g, escape ), 'g' );
        var escapedEndRegexp = new RegExp( endSymbol.replace( /./g, escape ), 'g' );

        function escape( ch ) {
          return '\\\\\\' + ch;
        }

        function unescapeText( text ) {
          return text.replace( escapedStartRegexp, startSymbol ).
            replace( escapedEndRegexp, endSymbol );
        }

        function $interpolate( text, mustHaveExpression, allOrNothing ) {
          // Provide a quick exit and simplified result
          // function for text with no interpolation
          if ( !text.length || text.indexOf( startSymbol ) === -1 ) {
            var constantInterp;
            if ( !mustHaveExpression ) {
              var unescapedText = unescapeText( text );
              constantInterp = valueFn( unescapedText );
              constantInterp.exp = text;
              constantInterp.expressions = [];
            }
            return constantInterp;
          }

          allOrNothing = !!allOrNothing;
          var startIndex;
          var endIndex;
          var index = 0;
          var expressions = [];
          var parseFns = [];
          var textLength = text.length;
          var exp;
          var concat = [];
          var expressionPositions = [];

          while ( index < textLength ) {
            if (
              ( ( startIndex = text.indexOf( startSymbol, index ) ) !== -1 ) &&
              ( ( endIndex = text.indexOf(
                endSymbol,
                startIndex + startSymbolLength
              ) ) !== -1 ) ) {
              if ( index !== startIndex ) {
                concat.push(
                  unescapeText( text.substring( index, startIndex ) )
                );
              }
              exp = text.substring( startIndex + startSymbolLength, endIndex );
              expressions.push( exp );
              parseFns.push( Parser( exp, parseStringifyInterceptor ) );
              index = endIndex + endSymbolLength;
              expressionPositions.push( concat.length );
              concat.push( '' );
            }
            else {
              // we did not find an interpolation, so we have to
              // add the remainder to the separators array
              if ( index !== textLength ) {
                concat.push( unescapeText( text.substring( index ) ) );
              }
              break;
            }
          }


          if ( !mustHaveExpression || expressions.length ) {
            var compute = function( values ) {
              for ( var i = 0, ii = expressions.length; i < ii; i++ ) {
                if ( allOrNothing && isUndefined( values[i] ) ) {
                  return; 
                }
                concat[expressionPositions[i]] = values[i];
              }
              return concat.join( '' );
            };

            return extend( function interpolationFn( context ) {
              var i = 0;
              var ii = expressions.length;
              var values = new Array( ii );

              try {
                for ( ; i < ii; i++ ) {
                  values[i] = parseFns[i]( context );
                }

                return compute( values );
              }
              catch ( err ) {
                console.error( 'interpolationFn error', text, err );
              }

            }, {
              // all of these properties are undocumented for now
              exp: text,
              //just for compatibility with regular watchers created via $watch
              expressions: expressions
            } );
          }

          function parseStringifyInterceptor( value ) {
            try {
              return allOrNothing && !isDefined( value ) ?
                value :
                JSON.stringify( value );
            }
            catch ( err ) {
              console.error( 'parseStringifyInterceptor error', err );
            }
          }
        }
        return $interpolate;
      }
    ]
  );
