import { module } from '../../unimatrix';
import { isDefined } from '../../functions/rtti';

module( 'Unimatrix' ).
  service( 'Interval',
    [ 'Unimatrix/Q',
      function( Q ) {
        var intervals = {};

        function interval( fn, delay, count ) {
          var setInterval = window.setInterval;
          var clearInterval = window.clearInterval;
          var iteration = 0;
          var deferred = Q.deferred();
          var promise = deferred.promise;

          count = isDefined( count ) ? count : 0;

          promise.then( null, null, fn );

          promise.$$intervalId = setInterval( function tick() {
            deferred.notify( iteration++ );
            if ( count > 0 && iteration >= count ) {
              deferred.resolve( iteration );
              clearInterval( promise.$$intervalId );
              delete intervals[ promise.$$intervalId ];
            }
          }, delay );

          intervals[ promise.$$intervalId ] = deferred;

          return promise;
        }

        interval.cancel = function( promise ) {
          if ( promise && promise.$$intervalId in intervals ) {
            intervals[promise.$$intervalId].reject( 'canceled' );
            window.clearInterval( promise.$$intervalId );
            delete intervals[ promise.$$intervalId ];
            return true;
          }
          return false;
        };

        return interval;
      }
    ]
  );