import { module } from '../../unimatrix';
import { each } from '../../functions/enumeration';
import { element } from '../../functions/element';
import { isDefined } from '../../functions/rtti';
import { noop } from '../../functions/function';

module( 'Unimatrix' ).
  service(
    'OperationCache',
    [
      'Unimatrix/Q', 'Unimatrix/ResourceParser',
      function( Q, ResourceParser ) {
        var operationCache = {};

        each( element( 'script' ), function( pageElement ) {
          var cacheKey = element( pageElement ).
            attr( 'data-unimatrix-operation-cache' );
          if ( cacheKey ) {
            try {
              operationCache[ cacheKey ] = function( content ) {
                return ResourceParser( JSON.parse( content.replace( /<!\[CDATA\[(.*?)\]\]>/g, '$1' ) ) ).
                  getResources();
              }( pageElement.innerHTML );
            }
            catch ( exception ) {
              noop();
            }
          }
        } );

        return function( key ) {
          var cachedResources = operationCache[ key ];
          if ( isDefined( cachedResources ) ) {
            return Q.when( cachedResources );
          }
          return null;
        };
      }
    ]
  );
