import { module } from '../../unimatrix';
import { isUndefined, isDefined } from '../../functions/rtti';

module( 'Unimatrix' ).
  service(
    'UserAgent',
    function() {

      function analyze( userAgent ) {

        function getFirstMatch( regex ) {
          var match = userAgent.match( regex );
          return ( match && match.length > 1 && match[ 1 ] ) || '';
        }

        var result = undefined;

        var iOSDevice = getFirstMatch( /(ipod|iphone|ipad)/i ).
          toLowerCase();
        var iOS = !!iOSDevice;
        var likeAndroid = /like android/i.test( userAgent );
        var android = !likeAndroid && /android/i.test( userAgent );
        var tablet = /tablet/i.test( userAgent ) || ( iOSDevice === 'ipad' );
        // var handheld = !tablet && (
        //   android ||
        //   iOSDevice === 'iphone' ||
        //   iOSDevice === 'ipod' ||
        //   /[^-]mobi/i.test( userAgent )
        // );
        // var windows = /Windows/.test( userAgent );
        // var desktop = !tablet && !handheld;

        var versionIdentifier = getFirstMatch( /version\/(\d+(\.\d+)?)/i );

        if ( /opera|opr/i.test( userAgent ) ) {
          result = {
            name: 'opera',
            version: versionIdentifier ||
               getFirstMatch( /(?:opera|opr)[\s/](\d+(\.\d+)?)/i )
          };
        }
        else if ( /windows phone/i.test( userAgent ) ) {
          result = {
            name: 'internet explorer',
            version: getFirstMatch( /iemobile\/(\d+(\.\d+)?)/i ),
            os: 'windows phone',
            osVersion: getFirstMatch( /windows phone (?:os)?\s?(\d+(\.\d+)*)/i )
          };
        }
        else if ( /msie|trident/i.test( userAgent ) ) {
          result = {
            name: 'internet explorer',
            version: getFirstMatch( /(?:msie |rv:)(\d+(\.\d+)?)/i )
          };
        }
        else if ( /edge/i.test( userAgent ) ) {
          result = {
            name: 'edge',
            version: getFirstMatch( /(?:edge)\/(\d+(?:\.\d+)?)/i )
          };
        }
        else if ( /chrome|crios|crmo/i.test( userAgent ) ) {
          result = {
            name: 'chrome',
            version: getFirstMatch( /(?:chrome|crios|crmo)\/(\d+(\.\d+)?)/i )
          };
        }
        else if ( /sailfish/i.test( userAgent ) ) {
          result = {
            deviceType: 'handheld',
            name: 'sailfish',
            version: getFirstMatch( /sailfish\s?browser\/(\d+(\.\d+)?)/i )
          };
        }
        else if ( /seamonkey\//i.test( userAgent ) ) {
          result = {
            deviceType: 'desktop',
            name: 'seamonkey',
            version: getFirstMatch( /seamonkey\/(\d+(\.\d+)?)/i )
          };
        }
        else if ( /firefox|iceweasel/i.test( userAgent ) ) {
          result = {
            name: 'firefox',
            version: getFirstMatch( /(?:firefox|iceweasel)[ /](\d+(\.\d+)?)/i )
          };
        }
        else if ( /silk/i.test( userAgent ) ) {
          result = {
            name: 'amazon silk',
            version: getFirstMatch( /silk\/(\d+(\.\d+)?)/i )
          };
        }
        else if ( /phantom/i.test( userAgent ) ) {
          result = {
            deviceType: 'headless',
            name: 'phantomjs',
            version: getFirstMatch( /phantomjs\/(\d+(\.\d+)?)/i )
          };
        }
        else if ( /blackberry|\bbb\d+/i.test( userAgent ) ||
                  /rim\stablet/i.test( userAgent ) ) {
          result = {
            deviceType: 'handheld',
            name: 'blackberry',
            version: versionIdentifier ||
            getFirstMatch( /blackberry[\d]+\/(\d+(\.\d+)?)/i ),
            os: 'blackberry',
            osVersion: getFirstMatch( /rim\stablet\sos\s(\d+(\.\d+)*)/i )
          };
        }
        else if ( /(web|hpw)os/i.test( userAgent ) ) {
          result = {
            deviceType: 'handheld',
            name: 'webos',
            version: versionIdentifier ||
            getFirstMatch( /w(?:eb)?osbrowser\/(\d+(\.\d+)?)/i ),
            os: 'webos',
            osVersion: getFirstMatch( /(?:web|hpw)os\/(\d+(\.\d+)*)/i )
          };
        }
        else if ( /bada/i.test( userAgent ) ) {
          result = {
            deviceType: 'handheld',
            name: 'dolfin',
            version: getFirstMatch( /dolfin\/(\d+(\.\d+)?)/i ),
            os: 'bada',
            osVersion: getFirstMatch( /bada\/(\d+(\.\d+)*)/i )
          };
        }
        else if ( /tizen/i.test( userAgent ) ) {
          result = {
            deviceType: 'handheld',
            name: 'tizen',
            version: getFirstMatch( /(?:tizen\s?)?browser\/(\d+(\.\d+)?)/i ) ||
              versionIdentifier,
            os: 'tizen',
            osVersion: getFirstMatch( /tizen[/\s](\d+(\.\d+)*)/i )
          };
        }
        else if ( /safari/i.test( userAgent ) ) {
          result = {
            name: 'safari',
            version: versionIdentifier
          };
        }
        else {
          result = {};
        }

        // set webkit or gecko flag for browsers based on these engines
        if ( /(apple)?webkit/i.test( userAgent ) ) {
          result.name = result.name || 'webkit';
          if ( !result.version && versionIdentifier ) {
            result.version = versionIdentifier;
          }
        }
        else if ( !result.opera && /gecko\//i.test( userAgent ) ) {
          result.name = result.name || 'gecko';
          result.version = result.version ||
            getFirstMatch( /gecko\/(\d+(\.\d+)?)/i );
        }

        // operating system
        var os = result.operatingSystem;
        var osVersion = '';
        if ( isUndefined( os ) ) {
          if ( iOS ) {
            os = 'ios';
            osVersion = getFirstMatch( /os (\d+([_\s]\d+)*) like mac os x/i );
            osVersion = osVersion.replace( /[_\s]/g, '.' );
          }
          else if ( android ) {
            os = 'android';
            osVersion = getFirstMatch( /android[ /-](\d+(\.\d+)*)/i );
          }
          else if ( /windows/i.test( userAgent ) ) {
            os = 'windows';
            // TODO: osVersion
          }
          else if ( /linux/i.test( userAgent ) ) {
            os = 'linux';
            // TODO: osVersion
          }
          else if ( /mac os/i.test( userAgent ) ) {
            os = 'mac-os';
            // TODO: osVersion
          }
          else {
            os = 'unknown';
          }
        }
        result.os = os;
        if ( osVersion ) {
          result.osVersion = osVersion;
        }

        // device type
        var deviceType = result.deviceType;
        if ( isUndefined( deviceType ) ) {
          var osMajorVersion = osVersion.split( '.' )[0];
          if ( tablet ||
               iOSDevice === 'ipad' ||
               ( android &&
                 ( osMajorVersion === 3 ||
                   ( osMajorVersion === 4 && !/Mobile/.test( userAgent ) )
                 )
               )
          ) {
            deviceType = 'tablet';
          }
          else if ( android || iOSDevice ) {
            deviceType = 'handheld';
          }
          else {
            deviceType = 'desktop';
          }
        }
        result.deviceType = deviceType || 'unknown';

        result.isPointerDevice = (
          (
            navigator.maxTouchPoints > 0 ||
            navigator.msMaxTouchPoints > 0
          ) &&
          ( result.os === 'windows' )
        ) ? true : false;

        return result;
      }

      var defaultUserAgentAnalysis = analyze(
        isDefined( window.navigator ) ?
          ( window.navigator.userAgent || '' ) :
          ''
      );

      function UserAgentFactory( userAgentString ) {
        this.analysis = isDefined( userAgentString ) ?
          analyze( userAgentString ) :
          defaultUserAgentAnalysis;
      }

      UserAgentFactory.prototype = {
        deviceType: function() {
          return this.analysis.deviceType;
        },

        name: function() {
          return this.analysis.name;
        },
        version: function() {
          return this.analysis.version;
        },

        desktop: function() {
          return this.analysis.deviceType === 'desktop';
        },
        handheld: function() {
          return this.analysis.deviceType === 'handheld';
        },
        tablet: function() {
          return this.analysis.deviceType === 'tablet';
        },
        isPointerDevice: function() {
          return this.analysis.isPointerDevice;
        },


        operatingSystem: function() {
          return this.analysis.os;
        },
        operatingSystemVersion: function() {
          return this.analysis.osVersion;
        },

        iOS: function() {
          return this.analysis.os === 'ios';
        },
        android: function() {
          return this.analysis.os === 'android';
        },
        isTouchDevice: function() {
          return (
            this.analysis.deviceType === 'handheld' ||
            this.analysis.deviceType === 'tablet' &&
            !this.analysis.isPointerDevice
          );
        }
      };

      return function( userAgentString ) {
        return new UserAgentFactory( userAgentString );
      };
    }
  );
