import { module } from '../../unimatrix';
import { extend } from '../../functions/object';
import { error } from '../../functions/error';

module( 'Unimatrix' ).
  service( 'Authorization',
    [ 'Unimatrix/Http',
      function( Http ) {
        var AuthorizationFactory = function( _authUrl, _config ) {
          var Authorization = function( authUrl, config ) {
            this.baseUrl = authUrl;
            this.credentials = extend( {}, config || {} );
          };

          Authorization.prototype = {
            constructor: Authorization,

            setUrl: function( url ) {
              this.baseUrl = url;
            },
            setConfig: function( config ) {
              this.credentials = extend( {}, config || {} );
            },
            getToken: function() {
              var reqConfig = {
                method: 'POST',
                url: this.baseUrl + '/token',
                data: {
                  client_id: this.credentials.id,
                  client_secret: this.credentials.secret,
                  grant_type: 'client_credentials'
                }
              };
              return Http( reqConfig ).
                then(
                  function( res ) {
                    var body = JSON.parse( res.body );

                    return {
                      expiresIn: body.expires_in,
                      accessToken: body.access_token
                    };
                  } ).
                catch(
                  function( err ) {
                    var errorResponse;

                    if ( err.status === 400 ) {
                      errorResponse = error( err.body );
                    }
                    else if ( err.status === 401 ) {
                      errorResponse = error( 'Invalid credentials.' );
                    }
                    else if ( err.status >= 500 ) {
                      errorResponse = error( 'Internal server error.' );
                    }
                    else {
                      errorResponse = error( 'Unknown error.' );
                    }

                    throw errorResponse;
                  }
                );
            }
          };

          return new Authorization( _authUrl, _config );
        };

        return AuthorizationFactory;
      }
    ]
  );
