import { module } from '../../unimatrix';
import { isString } from '../../functions/rtti';
import { error } from '../../functions/error';

var adaptors = {
  https: require( 'https' ),
  http: require( 'http' )
};

//  requestConfiguration:
//
//  method - {string}
//  url - {string}
//  params - {object}
//  body - {string|object}
//  headers – {Object}

module( 'Unimatrix' ).
  service(
    'Http',
    [
      'Unimatrix/URI', 'Unimatrix/Q',
      function( URI, Q ) {
        function isSuccess( status ) {
          return 200 <= status && status < 300;
        }

        function buildPath( url, params ) {
          if ( typeof params !== 'object' ||
            !Object.keys( params ).length ) {
            return url;
          }

          return url +
            ( ( url.indexOf( '?' ) === -1 ) ? '?' : '&' ) +
            URI.queryFromObject( params );
        }

        function Http( config ) {
          config = config || {};

          var parsedUrl = URI( config.url ) || '';
          var protocol = parsedUrl.getProtocol() || 'http';
          var host = parsedUrl.getHostname();
          var path = buildPath( parsedUrl.getPath(), config.params );
          var method = ( config.method || 'get' ).toUpperCase();
          var requestBody = config.data ?
            ( isString( config.data ) ?
              config.data :
              JSON.stringify( config.data ) ) :
            null;
          var deferred = Q.deferred();
          var promise = deferred.promise;

          var requestConfig = {
            host: host,
            path: '/' + path,
            method: method
          };
          if ( requestBody ) {
            requestConfig.headers = { 'Content-Type': 'application/json' };
          }
          var req = adaptors[ protocol ].
            request(
              requestConfig,
              function( res ) {
                var buffer = [];

                res.on(
                  'data',
                  function( data ) {
                    buffer.push( data );
                  }
                );
                res.on(
                  'end',
                  function() {
                    var body;

                    try {
                      body = Buffer.
                        concat( buffer ).
                        toString( 'utf8' );
                    }
                    catch ( _error ) {
                      body = '';
                    }

                    var result = {
                      body: body,
                      statusText: res.statusMessage,
                      status: res.statusCode,
                      headers: res.headers
                    };
                    isSuccess( res.statusCode ) ?
                      deferred.resolve( result ) :
                      deferred.reject( result );
                  }
                );
              }
            );

          req.on(
            'error',
            function( err ) {
              deferred.reject( error( 'Unable to make connection:', err ) );
            }
          );
          if ( requestBody ) {
            req.write( requestBody );
          }

          req.end();

          return promise;
        }
        return Http;
      }
    ]
  );
