import { isFunction } from '../../functions/rtti';
import { module } from '../../unimatrix';

function Q() {
  var deferredFactory = function() {
    var internalResolve;
    var internalReject;
    var internalPromise = new Promise(
      function( resolve, reject ) {
        internalResolve = resolve;
        internalReject = reject;
      }
    );
    var deferred = {
      resolve: function( val ) {
        internalResolve( val );
      },

      reject: function( reason ) {
        internalReject( reason );
      },

      promise: {
        then: function( callback, errback ) {
          return internalPromise.
            then( callback ).
            catch( errback );
        },
        catch: function( callback ) {
          return internalPromise.
            catch( callback );
        },
        finally: function( callback ) {
          return internalPromise.
            finally( callback );
        }
      }
    };

    return deferred;
  };

  var reject = function( reason ) {
    return Promise.reject( reason );
  };

  var when = function( value, callback, errback ) {
    var receivedPromise = value && typeof value.then === 'function';

    var wrappedCallback = function( value ) {
      try {
        return ( isFunction( callback ) ? callback : defaultCallback )( value );
      }
      catch ( e ) {
        return reject( e );
      }
    };

    var wrappedErrback = function( reason ) {
      try {
        return ( isFunction( errback ) ? errback : defaultErrback )( reason );
      }
      catch ( e ) {
        return reject( e );
      }
    };

    if ( receivedPromise ) {
      return value.
        then( wrappedCallback ).
        catch( wrappedErrback );
    }
    else {
      return wrappedCallback( value );
    }
  };

  function defaultCallback( value ) {
    return Promise.resolve( value );
  }

  function defaultErrback( reason ) {
    return Promise.reject( reason );
  }

  function all( promises ) {
    return Promise.all( promises );
  }

  return {
    deferred: deferredFactory,
    reject: reject,
    when: when,
    all: all
  };
}

module( 'Unimatrix' ).
  service(
    'Q',
    function() {
      return Q();
    }
  );
