import { module, injector } from '../../unimatrix';
import { pluralize } from '../../functions/string';
import { extend } from '../../functions/object';

// config: {
//   auth: {
//     url,
//     credentials: {
//       id,
//       secret
//     }
//   }
//   baseUrl
// }

module( 'Unimatrix' ).
  service( 'Wrapper',
    [ 'Unimatrix/Authorization', 'Unimatrix/Operation',
      function( Auth, Operation ) {
        var WrapperFactory = function( config ) {
          var wrappedAuth = Auth( config.auth.url, config.auth.credentials );
          var resources = injector().
            list( 'Resources' );
          Operation.baseUrl = config.baseUrl;

          var resourceMap = resources.reduce( function( acc, item ) {
            var resourceObj = {};
            var name = pluralize( item ).
              toLowerCase();
            resourceObj[name] = {};

            resourceObj[name].get = function( id, realm, token ) {
              return Operation( 'realms/' + realm + '/' + name + '/' + id ).
                where( { access_token: token } ).
                take();
            };

            resourceObj[name].write = function( realm, token, properties ) {
              return Operation( 'realms/' + realm + '/' + name ).
                where( { access_token: token } ).
                take().
                write( name, properties );
            };

            return extend( acc, resourceObj );
          }, {} );

          return { auth: wrappedAuth, resources: resourceMap };
        };

        return WrapperFactory;
      }
    ]
  );