import { module } from '../../../unimatrix';
import { isDate, isFunction, isString } from '../../../functions/rtti';

module( 'Unimatrix' ).
  service(
    'ResourceAttributeReflection',
    function() {
      var AttributeReflection = function( attribute ) {
        var self = this;
        // TO DO: Look up substitute for filter
        // var dateFilter = $filter( 'date' );

        self.name = attribute.name;
        self.alias = attribute.alias || attribute.name;
        self.type = attribute.type || 'string';
        self.format = attribute.format || null;
        self.persist = attribute.persist || true;

        self.from = function( value ) {
          if ( !value ) {
            return null;
          }
          else if ( self.type === 'string' ) {
            return value.toString();
          }
          else if ( self.type === 'date' || self.type === 'datetime' ) {
            var result = null;
            try {
              result = new Date( value );
            }
            catch ( e ) {
              console.error( e );
            }
            return result;
          }
          else if ( self.type === 'integer' ) {
            return parseInt( value.toString() );
          }
          else if ( self.type === 'float' ) {
            return parseFloat( value.toString() );
          }
          else if ( self.type === 'object' || self.type === 'json' ) {
            return value;
          }
          else if ( self.type === 'array' ) {
            return value;
          }
          else if ( self.type === 'boolean' ) {
            return value;
          }
          else {
            // TODO: Add provider based coersion
            throw 'ResourceAttribute: ' +
                  'An unknown type was given.';
          }
        };

        self.to = function( value ) {
          if ( isString( value ) ) {
            return value;
          }
          else if ( isDate( value ) ) {
            // return dateFilter( date, format );
          }
          else {
            if ( !isFunction( value.toString ) ) {
              throw 'ResourceAttribute: ' +
                    'An unsupported value was given.';
            }
            return value.toString();
          }
        };

      };

      return function( attribute ) {
        return new AttributeReflection( attribute );
      };
    }
  );