import { module } from '../../../unimatrix';
import { each } from '../../../functions/enumeration';
import { deepCopy } from '../../../functions/object';
import { isDefined, isFunction } from '../../../functions/rtti';

module( 'Unimatrix' ).
  service(
    'Resource',
    [
      'Unimatrix/ResourceReflection',
      'Unimatrix/ResourceAssociation',
      function( ResourceReflection, ResourceAssociation ) {

        var referenceReflection = ResourceReflection();

        var Resource = function( reflection ) {
          var resource = function( attributes, associations ) {
            var self = this;

            each(
              attributes,
              function( value, key ) {
                var attributeReflection =
                  reflection._attributesByNameOrAlias[ key ];
                if ( attributeReflection ) {
                  self[ attributeReflection.name ] = attributeReflection.
                    from( value );
                }
              }
            );

            if ( isDefined( associations ) ) {
              each(
                reflection._associations,
                function( associationReflection ) {
                  ResourceAssociation( associationReflection ).
                    install(
                      self,
                      associations[ associationReflection.property ||
                        associationReflection.name ]
                    );
                }
              );
            }

            for ( var property in reflection ) {
              if ( reflection.hasOwnProperty( property ) &&
                   !referenceReflection.hasOwnProperty( property ) ) {
                self[ property ] = reflection[ property ];
              }
            }

            self.duplicate = function() {
              var copiedResource = deepCopy( self );
              delete copiedResource.id;

              return copiedResource;
            };

            self.getKey = function() {
              return this[ reflection.key || 'id' ];
            };

            self.getAttribute = function( name ) {
              return attributes[ name ];
            };

            self.getAttributesByAlias = function() {
              var self = this;
              var attributes = {};
              each(
                reflection._attributes,
                function( attribute ) {
                  attributes[ attribute[ 'alias' ] || attribute[ 'name' ] ] =
                    self[ attribute[ 'name' ] ];
                }
              );
              return attributes;
            };

            return this;
          };

          resource.extend = function( definitionFactory ) {
            var extendedReflection = ResourceReflection( reflection );
            definitionFactory( extendedReflection );

            return Resource( extendedReflection );
          };

          resource.getAttributes = function() {
            return deepCopy( reflection._attributes );
          };

          return resource;
        };

        var ResourceFactory = function() {
          var self = this;

          self.define = function( definitionFactory ) {
            if ( !isFunction( definitionFactory ) ) {
              throw 'Resource.define requires a function argument.';
            }

            var reflection = ResourceReflection();
            definitionFactory( reflection );

            return Resource( reflection );
          };
        };

        return new ResourceFactory();
      }
    ]
  );