import { module } from '../../../unimatrix';
import {
  singularize,
  pluralize,
  camelCase
} from '../../../functions/string';
import { isArray, isDefined } from '../../../functions/rtti';
import { detect } from '../../../functions/enumeration';

module( 'Unimatrix' ).
  service(
    'ResourceAssociation',
    [ 'Unimatrix/Q',
      function( Q ) {
        var Association = function( reflection ) {
          var self = this;

          var name = reflection.name;
          var singularName = singularize( name );
          var pluralName = pluralize( singularName );

          var singularCapitalizedName = camelCase( singularName );
          var pluralCapitalizedName = camelCase( pluralName );

          self.install = function( into, associated ) {
            switch ( reflection.type ) {
            case 'BelongsTo': self.installBelongsTo( into, associated ); break;
            case 'HasMany': self.installHasMany( into, associated ); break;
            }
          };

          self.installBelongsTo = function( into, associated ) {
            into[ 'get' + singularCapitalizedName ] = function() {
              return Q.when(
                ( associated && associated.length > 0 ) ? associated[ 0 ] : null
              );
            };
          };

          self.installHasOne = function( into, associated ) {
            into[ 'get' + singularCapitalizedName ] = function() {
              return Q.when( isArray( associated ) ?
                associated.first :
                null
              );
            };
          };

          self.installHasMany = function( into, associated ) {
            into[ 'get' + pluralCapitalizedName ] = function() {
              return Q.when( associated );
            };

            into[ 'get' + singularCapitalizedName + 'By' ] =
              function( fieldName , fieldValue ) {
                return into[ 'get' + pluralCapitalizedName ]().
                  then(
                    function( collection ) {
                      var resource = detect(
                        collection,
                        function( item ) {
                          return item[ fieldName ] === fieldValue;
                        }
                      );

                      return isDefined( resource ) ?
                        Q.when( resource ) : Q.reject();
                    }
                  );
              };

          };
        };

        return function( reflection ) {
          return new Association( reflection );
        };
      }
    ]
  );
