import { module } from '../../../unimatrix';
import {
  isArray,
  isString,
  isObject,
  isDefined
} from '../../../functions/rtti';
import { each } from '../../../functions/enumeration';
import { deepCopy } from '../../../functions/object';
import { equals } from '../../../functions/equivalence';

module( 'Unimatrix' ).
  service(
    'ResourceReflection',
    [
      'Unimatrix/ResourceAttributeReflection',
      'Unimatrix/ResourceAssociationReflection',
      function( ResourceAttributeReflection, ResourceAssociationReflection ) {
        var Reflection = function( existingReflection ) {
          existingReflection = existingReflection || {};
          var self = this;

          self.key = existingReflection || 'id';
          self.url = existingReflection || null;
          self.include = existingReflection || null;
          self._attributes = [];
          self._attributesByNameOrAlias = {};
          self._associations = [];
          if ( !equals( existingReflection, {} ) ) {
            self._attributes = deepCopy( existingReflection._attributes );
            self._attributesByNameOrAlias = deepCopy(
              existingReflection._attributesByNameOrAlias
            );
            self._associations = deepCopy( existingReflection._associations );
          }

          self.attributes = function( attributes ) {
            if ( !isArray( attributes ) ) {
              throw 'ResourceReflection: An attributes array is required.';
            }
            each(
              attributes,
              function( attribute ) {
                var reflection;

                if ( isString( attribute ) ) {
                  reflection = ResourceAttributeReflection(
                    { name: attribute }
                  );
                  self._attributesByNameOrAlias[ attribute ] = reflection;
                  self._attributes.push( reflection );
                }
                else if ( isObject( attribute ) ) {
                  if ( !attribute[ 'name' ] ) {
                    throw 'ResourceReflection: ' +
                          'An attribute must have a name.';
                  }
                  reflection = ResourceAttributeReflection( attribute );
                  self._attributes.push( reflection );
                  self._attributesByNameOrAlias[
                    attribute[ 'name' ]
                  ] = reflection;
                  if ( isDefined( attribute[ 'alias' ] ) ) {
                    self._attributesByNameOrAlias[
                      attribute[ 'alias' ]
                    ] = reflection;
                  }
                }
                else {
                  throw 'ResourceReflection: ' +
                        'An attribute must be a string or object.';
                }
              }
            );
          };

          self.belongsTo = function( name, options ) {
            self._associations.push(
              new ResourceAssociationReflection(
                'BelongsTo',
                name,
                options || {}
              )
            );
          };

          self.hasOne = function( name, options ) {
            self._associations.push(
              new ResourceAssociationReflection(
                'HasOne',
                name,
                options || {}
              )
            );
          };

          self.hasMany = function( name, options ) {
            self._associations.push(
              new ResourceAssociationReflection(
                'HasMany',
                name,
                options || {}
              )
            );
          };

          self.scope = function( name, implementation ) {
            self.scopes[ name ] = implementation;
          };

        };

        return function( existingReflection ) {
          return new Reflection( existingReflection );
        };
      }
    ]
  );
