module.exports = function( operation ) {
  describe( 'operation service', () => {
    const url = '/realms/1234/artifacts';

    describe( 'where', () => {
      test( 'should include given parameters', () => {
        const opTest = operation( url ).
          where( { uuid: 12345 } );

        expect( opTest.options.params ).
          toMatchObject( { uuid: 12345 } );

        opTest.where( { hats: 'off' } );

        expect( opTest.options.params ).
          toMatchObject( { uuid: 12345, hats: 'off' } );
      } );
    } );

    describe( 'order', () => {
      test( 'should add sort by and direction parameters', () => {
        const opTest = operation( url ).
          order( 'name', 'ascending' );

        expect( opTest.options.params ).
          toMatchObject( { sort_by: 'name', sort_direction: 'ascending' } );
      } );
    } );

    describe( 'include', () => {
      test( 'should accept strings', () => {
        const opTest = operation( url ).
          include( 'stockings' );

        expect( opTest.options.params.include ).
          toMatchObject( { stockings: true } );
      } );
      test( 'should accept arrays', () => {
        const opTest = operation( url ).
          include( 'stockings', 'shoes' );

        expect( opTest.options.params.include ).
          toMatchObject( { stockings: true, shoes: true } );
      } );
      test( 'should accept objects', () => {
        const opTest = operation( url ).
          include( { iguanas: 'with hats' } );

        expect( opTest.options.params.include ).
          toMatchObject( { iguanas: { 'with hats': true } } );
      } );
    } );

    describe( 'limit', () => {
      test( 'should add a limit param', () => {
        const opTest = operation( url ).
          limit( 6 );

        expect( opTest.options.params.count ).
          toBe( 6 );
      } );
    } );

    describe( 'offset', () => {
      test( 'should add an offset param', () => {
        const opTest = operation( url ).
          offset( 5 );

        expect( opTest.options.params.offset ).
          toBe( 5 );
      } );
    } );

    describe( 'find', () => {
      test( 'should add an id param and update the result type', () => {
        const opTest = operation( url ).
          find( 51 );

        expect( opTest.options.params.id ).
          toBe( 51 );
        expect( opTest.options.typeOfResult ).
          toBe( 'object' );
      } );
    } );

    describe( 'take', () => {
      test( 'should update the result type', () => {
        const opTest = operation( url ).
          take();

        expect( opTest.options.typeOfResult ).
          toBe( 'object' );
      } );
    } );
  } );
};
