import {
  lowerCase,
  upperCase,
  camelCase,
  lowerCamelCase,
  dashCase,
  pluralize,
  singularize,
  digest,
  padLeft,
  padRight
} from '../../src/functions/string';

describe( 'string', () => {
  describe( 'lowerCase should', () => {
    test( 'lowercase a string', () => {
      expect( lowerCase( 'CAPTAIN CAPSLOCK' ) ).
        toBe( 'captain capslock' );
    } );
  } );

  describe( 'upperCase should', () => {
    test( 'uppercase a string', () => {
      expect( upperCase( 'lieutenant lowercase' ) ).
        toBe( 'LIEUTENANT LOWERCASE' );
    } );
  } );

  describe( 'camelCase should', () => {
    test( 'camelcase a string', () => {
      expect( camelCase( 'corporal camelcase' ) ).
        toBe( 'CorporalCamelcase' );
    } );
  } );

  describe( 'lowerCamelCase should', () => {
    test( 'camelcase a string and leave the first letter lowercase', () => {
      expect( lowerCamelCase( 'corporal camelcase' ) ).
        toBe( 'corporalCamelcase' );
    } );
  } );

  describe( 'dashCase should', () => {
    test( 'dashcase a string', () => {
      expect( dashCase( 'doctor dashcase' ) ).
        toBe( 'doctor-dashcase' );
    } );
  } );

  describe( 'pluralize should', () => {
    test( 'pluralize a string that doesn\'t end in y', () => {
      expect( pluralize( 'prince plural' ) ).
        toBe( 'prince plurals' );
    } );

    test( 'pluralize a string that ends in y', () => {
      expect( pluralize( 'prince plurality' ) ).
        toBe( 'prince pluralities' );
    } );
  } );

  describe( 'singularize should', () => {
    test( 'singularize a string that doesn\'t end in y', () => {
      expect( singularize( 'prince plurals' ) ).
        toBe( 'prince plural' );
    } );

    test( 'singularize a string that ends in y', () => {
      expect( singularize( 'prince pluralities' ) ).
        toBe( 'prince plurality' );
    } );
  } );

  describe( 'digest should', () => {
    test( 'digest a string', () => {
      expect( digest( 'diver digest' ) ).
        toBe( 1812298837 );
    } );
  } );

  describe( 'padleft should', () => {
    test(
      'pad a string to the given size by adding characters to the right',
      () => {
        expect( padLeft( 'bob', 8, '$' ) ).
          toBe( '$$$$$bob' );
        expect( padLeft( undefined, 8, '^' ) ).
          toBe( '^^^^^^^^' );
        expect( padLeft( 5, 2, '0' ) ).
          toBe( '05' );
        expect( padLeft( undefined, 2, '0' ) ).
          toBe( '00' );
        expect( padLeft( 5, undefined, '0' ) ).
          toBe( '5' );
        expect( padLeft( 5, 2, undefined ) ).
          toBe( ' 5' );
        expect( padLeft( null, 'seven', Infinity ) ).
          toBe( 'seven' );
        expect( padLeft( '000', 6, '000' ) ).
          toBe( '000000' );
        expect( padLeft( "'soon'", 8 ) ).
          toBe( "  'soon'" );
        expect( padLeft( '', 9, '_' ) ).
          toBe( '_________' );
      } );
  } );

  describe( 'padright should', () => {
    test(
      'pad a string to the given size by adding characters to the left',
      () => {
        expect( padRight( 'bob', 8, '$' ) ).
          toBe( 'bob$$$$$' );
        expect( padRight( undefined, 8, '^' ) ).
          toBe( '^^^^^^^^' );
        expect( padRight( 5, 2, '0' ) ).
          toBe( '50' );
        expect( padRight( undefined, 2, '0' ) ).
          toBe( '00' );
        expect( padRight( 5, undefined, '0' ) ).
          toBe( '5' );
        expect( padRight( 5, 2, undefined ) ).
          toBe( '5 ' );
        expect( padRight( null, 'seven', Infinity ) ).
          toBe( 'seven' );
        expect( padRight( '000', 6, '000' ) ).
          toBe( '000000' );
        expect( padRight( "'soon'", 8 ) ).
          toBe( "'soon'  " );
        expect( padRight( '', 9, '_' ) ).
          toBe( '_________' );
      } );
  } );
} );
